import { EventEmitter, Directive, ElementRef, Output, HostListener, Injectable, ɵɵdefineInjectable, Component, Input, ViewContainerRef, ComponentFactoryResolver, ViewChild, forwardRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { BehaviorSubject, noop } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ClickOutsideDirective {
    /**
     * @param {?} _elementRef
     */
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        this.appClickOutside = new EventEmitter();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        /** @type {?} */
        const targetElementPath = typeof event.composedPath === 'function' ? event.composedPath() : this._getComposedPath(event.target);
        /** @type {?} */
        const elementRefInPath = targetElementPath.find((/**
         * @param {?} e
         * @return {?}
         */
        e => e === this._elementRef.nativeElement));
        if (!elementRefInPath) {
            this.appClickOutside.emit(event);
        }
    }
    /**
     * @private
     * @param {?} node
     * @return {?}
     */
    _getComposedPath(node) {
        /** @type {?} */
        let parent;
        if (node.parentNode) {
            parent = node.parentNode;
        }
        else if (node.host) {
            parent = node.host;
        }
        else if (node.defaultView) {
            parent = node.defaultView;
        }
        if (parent !== undefined) {
            return [node].concat(this._getComposedPath(parent));
        }
        return [node];
    }
    ;
}
ClickOutsideDirective.decorators = [
    { type: Directive, args: [{
                selector: '[clickOutside]'
            },] }
];
/** @nocollapse */
ClickOutsideDirective.ctorParameters = () => [
    { type: ElementRef }
];
ClickOutsideDirective.propDecorators = {
    appClickOutside: [{ type: Output }],
    onClick: [{ type: HostListener, args: ['document:click', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableCellService {
    constructor() { }
    /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    getTableCellValue(tableRowData, tableColumnDefinition) {
        if (!tableRowData || !tableColumnDefinition)
            return;
        /** @type {?} */
        const fields = tableColumnDefinition.field.split('.');
        /** @type {?} */
        let cellValue = tableRowData;
        for (let i = 0; i < fields.length; i++) {
            /** @type {?} */
            const key = fields[i];
            if (cellValue[key])
                cellValue = cellValue[key];
            else
                return undefined;
        }
        if (tableColumnDefinition.valueFormatter) {
            cellValue = tableColumnDefinition.valueFormatter(cellValue, tableRowData, tableColumnDefinition);
        }
        return { value: cellValue };
    }
}
PlTableCellService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
PlTableCellService.ctorParameters = () => [];
/** @nocollapse */ PlTableCellService.ngInjectableDef = ɵɵdefineInjectable({ factory: function PlTableCellService_Factory() { return new PlTableCellService(); }, token: PlTableCellService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableComponent {
    /**
     * @param {?} plTableCellService
     */
    constructor(plTableCellService) {
        this.plTableCellService = plTableCellService;
        this._tableData = [];
        this.appliedTableOptions = {};
        this.selectedTableRows = [];
        this.tableOptions = {};
        this.tableRowClicked = new EventEmitter();
        this.tableRowSelectionChanged = new EventEmitter();
    }
    /**
     * @param {?} tableData
     * @return {?}
     */
    set tableData(tableData) {
        if (this.tableColumnsDefinitions) {
            this._tableData = tableData ? tableData : [];
            this.applyTableOptions(this.tableOptions);
        }
    }
    /**
     * @return {?}
     */
    get tableData() {
        return this._tableData;
    }
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    applyTableOptions(options) {
        this.appliedTableOptions.sortBy = options.sortBy || null;
        this.appliedTableOptions.sortOrder = options.sortOrder || 'asc';
        this.appliedTableOptions.selectableRows = options.selectableRows || false;
        this.appliedTableOptions.selectOnRowClick = options.selectOnRowClick || false;
        this.sortTableData();
    }
    /* Sort table rows */
    /**
     * @param {?} sortOptions
     * @return {?}
     */
    onSortChange(sortOptions) {
        if (!sortOptions)
            return;
        this.appliedTableOptions.sortBy = sortOptions.tableColumnKey;
        this.appliedTableOptions.sortOrder = sortOptions.sortOrder;
        this.sortTableData();
    }
    /**
     * @private
     * @return {?}
     */
    sortTableData() {
        this.tableData.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => {
            /** @type {?} */
            const orderModifyer = this.appliedTableOptions.sortOrder === 'asc' ? 1 : -1;
            /** @type {?} */
            const tableColumnDefinition = this.tableColumnsDefinitions.find((/**
             * @param {?} item
             * @return {?}
             */
            item => item.field === this.appliedTableOptions.sortBy));
            /** @type {?} */
            const tableCellA = this.plTableCellService.getTableCellValue(a, tableColumnDefinition);
            /** @type {?} */
            const tableCellB = this.plTableCellService.getTableCellValue(b, tableColumnDefinition);
            /** @type {?} */
            let tableCellAValue = tableCellA ? tableCellA.value : 0;
            /** @type {?} */
            let tableCellBValue = tableCellB ? tableCellB.value : 0;
            if (typeof tableCellAValue === 'string') {
                tableCellAValue = tableCellAValue.toLowerCase();
            }
            if (typeof tableCellBValue === 'string') {
                tableCellBValue = tableCellBValue.toLowerCase();
            }
            if (tableCellAValue > tableCellBValue)
                return 1 * orderModifyer;
            if (tableCellAValue < tableCellBValue)
                return -1 * orderModifyer;
            return 0;
        }));
    }
    /* Selected table rows */
    /**
     * @param {?} tableRow
     * @return {?}
     */
    onSelectedTableRowsToggle(tableRow) {
        if (this.selectedTableRows.indexOf(tableRow) > -1) {
            this.onSelectedTableRowsRemove(tableRow);
        }
        else {
            this.onSelectedTableRowsAdd(tableRow);
        }
    }
    /**
     * @param {?} tableRow
     * @return {?}
     */
    onSelectedTableRowsAdd(tableRow) {
        this.selectedTableRows.push(tableRow);
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    }
    /**
     * @param {?} tableRow
     * @return {?}
     */
    onSelectedTableRowsRemove(tableRow) {
        /** @type {?} */
        const rowIndex = this.selectedTableRows.indexOf(tableRow);
        if (rowIndex > -1) {
            this.selectedTableRows.splice(rowIndex, 1);
            this.tableRowSelectionChanged.emit(this.selectedTableRows);
        }
    }
    /**
     * @return {?}
     */
    onSelectedTableRowsAddAll() {
        this.selectedTableRows = this.tableData;
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    }
    /**
     * @return {?}
     */
    onSelectedTableRowsRemoveAll() {
        this.selectedTableRows = [];
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    }
    /* Table events */
    /**
     * @param {?} event
     * @param {?} tableRowData
     * @param {?} tableRowIndex
     * @return {?}
     */
    onTableRowClick(event, tableRowData, tableRowIndex) {
        this.tableRowClicked.emit({ event, tableRowData, tableRowIndex });
        // select row if "selectOnRowClick" option is set true
        if (this.appliedTableOptions.selectableRows
            && this.appliedTableOptions.selectOnRowClick) {
            this.onSelectedTableRowsToggle(tableRowData);
        }
    }
}
PlTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table',
                template: "<pl-table-header\n    class=\"pl-table__header\"\n    [tableOptions]=\"appliedTableOptions\"\n    [tableColumnDefinitions]=\"tableColumnsDefinitions\"\n    [allTabelRowsAreSelected]=\"tableData.length === selectedTableRows.length\"\n    (sortChange)=\"onSortChange($event)\"\n    (allRowsSelected)=\"onSelectedTableRowsAddAll()\"\n    (AllRowsUnselected)=\"onSelectedTableRowsRemoveAll()\">\n</pl-table-header>\n<pl-table-row \n    class=\"pl-table__row\"\n    *ngFor=\"let tableRowData of tableData; let i = index\"\n    [tableOptions]=\"appliedTableOptions\"\n    [tableColumnDefinitions]=\"tableColumnsDefinitions\"\n    [tableRowData]=\"tableRowData\"\n    [selectedTableRows]=\"selectedTableRows\"\n    (rowSelected)=\"onSelectedTableRowsAdd($event)\"\n    (rowUnselected)=\"onSelectedTableRowsRemove($event)\"\n    (click)=\"onTableRowClick($event, tableRowData, i)\">\n</pl-table-row>",
                host: { '[class.pl-table]': 'true' },
                providers: [PlTableCellService],
                styles: [""]
            }] }
];
/** @nocollapse */
PlTableComponent.ctorParameters = () => [
    { type: PlTableCellService }
];
PlTableComponent.propDecorators = {
    tableOptions: [{ type: Input }],
    tableColumnsDefinitions: [{ type: Input }],
    tableRowClicked: [{ type: Output }],
    tableRowSelectionChanged: [{ type: Output }],
    tableData: [{ type: Input, args: ['tableData',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableRowComponent {
    /**
     * @param {?} plTableCellService
     */
    constructor(plTableCellService) {
        this.plTableCellService = plTableCellService;
        this.rowSelected = new EventEmitter();
        this.rowUnselected = new EventEmitter();
    }
    /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    getCellValue(tableRowData, tableColumnDefinition) {
        return this.plTableCellService.getTableCellValue(tableRowData, tableColumnDefinition);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTableRowSelect(event) {
        event.stopPropagation();
        if (this.isSelectedTableRow())
            this.rowUnselected.emit(this.tableRowData);
        else
            this.rowSelected.emit(this.tableRowData);
    }
    /**
     * @return {?}
     */
    isSelectedTableRow() {
        return this.selectedTableRows.indexOf(this.tableRowData) > -1;
    }
}
PlTableRowComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table-row',
                template: "<div class=\"pl-table__row__selector\" *ngIf=\"tableOptions.selectableRows\">\n    <input type=\"checkbox\" (click)=\"onTableRowSelect($event)\" [checked]=\"isSelectedTableRow()\">\n</div>\n<pl-table-cell \n    class=\"pl-table__cell\"\n    *ngFor=\"let tableColumnDefinition of tableColumnDefinitions; let i = index\"\n    [tableCellData]=\"getCellValue(tableRowData, tableColumnDefinition)\"\n    [tableColumnDefinition]=\"tableColumnDefinition\">\n</pl-table-cell>",
                styles: [":host{display:-webkit-box;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;flex-direction:row;flex-wrap:nowrap;width:100%}"]
            }] }
];
/** @nocollapse */
PlTableRowComponent.ctorParameters = () => [
    { type: PlTableCellService }
];
PlTableRowComponent.propDecorators = {
    tableOptions: [{ type: Input }],
    tableColumnDefinitions: [{ type: Input }],
    tableRowData: [{ type: Input }],
    selectedTableRows: [{ type: Input }],
    rowSelected: [{ type: Output }],
    rowUnselected: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableCellDirective {
    /**
     * @param {?} viewContainerRef
     */
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
}
PlTableCellDirective.decorators = [
    { type: Directive, args: [{
                selector: '[pl-table-cell]'
            },] }
];
/** @nocollapse */
PlTableCellDirective.ctorParameters = () => [
    { type: ViewContainerRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableCellRendererComponent {
    constructor() { }
}
PlTableCellRendererComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table-cell-renderer',
                template: "{{ tableCellData?.value }}",
                styles: [""]
            }] }
];
/** @nocollapse */
PlTableCellRendererComponent.ctorParameters = () => [];
PlTableCellRendererComponent.propDecorators = {
    tableCellData: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableCellComponent {
    /**
     * @param {?} componentFactoryResolver
     */
    constructor(componentFactoryResolver) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.defaultTableCellRenderer = PlTableCellRendererComponent;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.loadTableCellRenderer();
    }
    /**
     * @private
     * @return {?}
     */
    loadTableCellRenderer() {
        /** @type {?} */
        const tableCellRenderer = this.tableColumnDefinition.tableCellRenderer || this.defaultTableCellRenderer;
        /** @type {?} */
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(tableCellRenderer);
        /** @type {?} */
        const viewContainerRef = this.tableCellRenderer.viewContainerRef;
        viewContainerRef.clear();
        /** @type {?} */
        const componentRef = viewContainerRef.createComponent(componentFactory);
        ((/** @type {?} */ (componentRef.instance))).tableCellData = this.tableCellData;
    }
}
PlTableCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table-cell',
                template: "<ng-template pl-table-cell></ng-template>",
                styles: [":host{-webkit-box-flex:1;flex-grow:1;width:20rem}"]
            }] }
];
/** @nocollapse */
PlTableCellComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
PlTableCellComponent.propDecorators = {
    tableCellData: [{ type: Input }],
    tableColumnDefinition: [{ type: Input }],
    tableCellRenderer: [{ type: ViewChild, args: [PlTableCellDirective, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableHeaderComponent {
    constructor() {
        this.sortChange = new EventEmitter();
        this.allRowsSelected = new EventEmitter();
        this.AllRowsUnselected = new EventEmitter();
    }
    /**
     * @param {?} eventData
     * @return {?}
     */
    onSortChange(eventData) {
        this.sortChange.emit(eventData);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTableRowSelectAll(event) {
        event.stopPropagation();
        if (!this.allTabelRowsAreSelected)
            this.allRowsSelected.emit();
        else
            this.AllRowsUnselected.emit();
    }
}
PlTableHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table-header',
                template: "<div class=\"pl-table__header__row-selector\" *ngIf=\"tableOptions.selectableRows\">\n    <input type=\"checkbox\" (click)=\"onTableRowSelectAll($event)\" [checked]=\"allTabelRowsAreSelected\">\n</div>\n<pl-table-header-cell \n    class=\"pl-table__header__cell \"\n    *ngFor=\"let tableColumnDefinition of tableColumnDefinitions; let i = index\"\n    [tableOptions]=\"tableOptions\"\n    [tableColumnDefinition]=\"tableColumnDefinition\"\n    (sortChange)=\"onSortChange($event)\">\n</pl-table-header-cell>",
                styles: [":host{display:-webkit-box;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;flex-direction:row;flex-wrap:nowrap;width:100%}"]
            }] }
];
/** @nocollapse */
PlTableHeaderComponent.ctorParameters = () => [];
PlTableHeaderComponent.propDecorators = {
    tableOptions: [{ type: Input }],
    tableColumnDefinitions: [{ type: Input }],
    allTabelRowsAreSelected: [{ type: Input }],
    sortChange: [{ type: Output }],
    allRowsSelected: [{ type: Output }],
    AllRowsUnselected: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlTableHeaderCellComponent {
    constructor() {
        this.sortChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    onLabelClick() {
        if (this.tableColumnDefinition.sortable) {
            /** @type {?} */
            const sortOrder = this.tableOptions.sortBy === this.tableColumnDefinition.field ? this.tableOptions.sortOrder === 'asc' ? 'desc' : 'asc' : 'asc';
            this.sortChange.emit({ tableColumnKey: this.tableColumnDefinition.field, sortOrder });
        }
    }
}
PlTableHeaderCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-table-header-cell',
                template: "<span \n    class=\"pl-table__header__cell__label\"\n    [ngClass]=\"{\n        'pl-table__header__cell__label--sortable': tableColumnDefinition.sortable,\n        'pl-table__header__cell__label--sorted': tableColumnDefinition.field === tableOptions.sortBy,\n        'pl-table__header__cell__label--sorted-asc': tableColumnDefinition.field === tableOptions.sortBy && tableOptions.sortOrder === 'asc',\n        'pl-table__header__cell__label--sorted-desc': tableColumnDefinition.field === tableOptions.sortBy && tableOptions.sortOrder === 'desc'\n    }\"\n    (click)=\"onLabelClick()\">\n        {{ tableColumnDefinition.title }}\n</span>",
                styles: [":host{-webkit-box-flex:1;flex-grow:1;width:20rem}"]
            }] }
];
/** @nocollapse */
PlTableHeaderCellComponent.ctorParameters = () => [];
PlTableHeaderCellComponent.propDecorators = {
    tableOptions: [{ type: Input }],
    tableColumnDefinition: [{ type: Input }],
    sortChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlDatepickerService {
    constructor() {
        this.selectedDate$ = new BehaviorSubject(this.selectedDate);
        this.currentMonth$ = new BehaviorSubject(this.currentMonth);
        this.setCurrentMonth(this.getFirstDayOfMonth(new Date()));
    }
    /**
     * @param {?} date
     * @return {?}
     */
    setSelectedDate(date) {
        this.selectedDate = date;
        this.selectedDate$.next(this.selectedDate);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    setCurrentMonth(date) {
        this.currentMonth = this.getFirstDayOfMonth(date);
        this.currentMonth$.next(this.currentMonth);
    }
    /**
     * @param {?} month
     * @return {?}
     */
    getMonthMatrix(month) {
        if (!this.isTypeOfDate(month))
            return;
        /** @type {?} */
        const firstDayWeekIndex = this.adjustWeekIndex(this.getFirstDayOfMonth(month).getDay());
        /** @type {?} */
        const matrixStartDate = new Date(month.getFullYear(), month.getMonth(), 1 - firstDayWeekIndex);
        /** @type {?} */
        let daysOfTheMonth = [];
        /** @type {?} */
        let days = [];
        for (let i = 0; i <= 42; i++) {
            /** @type {?} */
            const date = new Date(matrixStartDate.getFullYear(), matrixStartDate.getMonth(), matrixStartDate.getDate() + i);
            days.push(date);
            if ((i + 1) % 7 === 0) {
                daysOfTheMonth.push({
                    no: this.getWeek(date),
                    days
                });
                days = [];
            }
        }
        return daysOfTheMonth;
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isTypeOfDate(date) {
        return Object.prototype.toString.call(date) === '[object Date]';
    }
    /**
     * @param {?} date
     * @return {?}
     */
    getFirstDayOfMonth(date) {
        if (!this.isTypeOfDate(date)) {
            date = new Date();
        }
        return new Date(date.getFullYear(), date.getMonth(), 1);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    getLastDayOfMonth(date) {
        if (!this.isTypeOfDate(date)) {
            date = new Date();
        }
        return new Date(date.getFullYear(), date.getMonth() + 1, 0);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isInCurrentMonth(date) {
        return date.getMonth() === this.currentMonth.getMonth();
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isSelectedDate(date) {
        return this.isSameDay(date, this.selectedDate);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isToday(date) {
        return this.isSameDay(date, new Date());
    }
    /**
     * @param {?} date1
     * @param {?} date2
     * @return {?}
     */
    isSameDay(date1, date2) {
        if (!this.isTypeOfDate(date1) || !this.isTypeOfDate(date2))
            return;
        return date1.getDate() == date2.getDate() &&
            date1.getMonth() == date2.getMonth() &&
            date1.getFullYear() == date2.getFullYear();
    }
    /**
     * @param {?} index
     * @return {?}
     */
    adjustWeekIndex(index) {
        /** @type {?} */
        const offset = 1;
        return index < offset ? 7 - offset : index - offset;
    }
    /**
     * @param {?} _date
     * @return {?}
     */
    getWeek(_date) {
        if (!this.isTypeOfDate(_date))
            return 0;
        /** @type {?} */
        let date = new Date(_date.getTime());
        date.setHours(0, 0, 0, 0);
        date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
        /** @type {?} */
        const week1 = new Date(date.getFullYear(), 0, 4);
        return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000 - 3 + (week1.getDay() + 6) % 7) / 7);
    }
}
PlDatepickerService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
PlDatepickerService.ctorParameters = () => [];
/** @nocollapse */ PlDatepickerService.ngInjectableDef = ɵɵdefineInjectable({ factory: function PlDatepickerService_Factory() { return new PlDatepickerService(); }, token: PlDatepickerService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlDatepickerComponent {
    /**
     * @param {?} plDatepickerService
     */
    constructor(plDatepickerService) {
        this.plDatepickerService = plDatepickerService;
        this.showOverlay = false;
        this.appliedOptions = {};
        this.datepickerOptions = {};
        this.propagateChange = (/**
         * @param {?} date
         * @return {?}
         */
        (date) => date);
        this.propagateTouch = noop;
        this.selectedDateSubscription = this.plDatepickerService.selectedDate$.subscribe((/**
         * @param {?} date
         * @return {?}
         */
        date => {
            this.onDateSelect(date);
        }));
        this.applyOptions(this.datepickerOptions);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.applyOptions(this.datepickerOptions);
    }
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    applyOptions(options) {
        this.appliedOptions.dateFormat = options.dateFormat || 'dd.MM.yyyy';
    }
    /**
     * @param {?} date
     * @return {?}
     */
    onDateSelect(date) {
        this.model = date;
        this.propagateChange(this.model);
        if (this.model) {
            this.showOverlay = false;
        }
    }
    /**
     * @return {?}
     */
    toggleDatepickerOverlay() {
        this.showOverlay = !this.showOverlay;
        this.propagateTouch();
    }
    /**
     * @return {?}
     */
    hideOverlay() {
        this.showOverlay = false;
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        this.plDatepickerService.setSelectedDate(obj || null);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.propagateTouch = fn;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.selectedDateSubscription.unsubscribe();
    }
}
PlDatepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-datepicker',
                template: "<div class=\"pl-datepicker\" (clickOutside)=\"hideOverlay()\">\n    <span class=\"pl-datepicker__model\" (click)=\"toggleDatepickerOverlay()\">{{ model | date:appliedOptions.dateFormat }}</span>\n    <pl-datepicker-overlay *ngIf=\"showOverlay\"></pl-datepicker-overlay>\n</div>",
                providers: [
                    PlDatepickerService,
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => PlDatepickerComponent)),
                        multi: true,
                    }
                ],
                styles: [""]
            }] }
];
/** @nocollapse */
PlDatepickerComponent.ctorParameters = () => [
    { type: PlDatepickerService }
];
PlDatepickerComponent.propDecorators = {
    datepickerOptions: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlDatepickerOverlayComponent {
    /**
     * @param {?} plDatepickerService
     */
    constructor(plDatepickerService) {
        this.plDatepickerService = plDatepickerService;
        this.selectedDateSubscription = this.plDatepickerService.selectedDate$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        date => this.model = date));
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        date => this.currentMonth = date));
    }
    /**
     * @return {?}
     */
    getDaysInMonth() {
        return this.plDatepickerService.getMonthMatrix(this.currentMonth);
    }
    /**
     * @return {?}
     */
    onNavigateToNextMonth() {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() + 1, 1));
    }
    /**
     * @return {?}
     */
    onNavigateToPreviousMonth() {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() - 1, 1));
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isInCurrentMonth(date) {
        return this.plDatepickerService.isInCurrentMonth(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isSelectedDate(date) {
        return this.plDatepickerService.isSelectedDate(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isToday(date) {
        return this.plDatepickerService.isToday(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    onDateSelect(date) {
        this.plDatepickerService.setSelectedDate(date);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.selectedDateSubscription.unsubscribe();
        this.currentMonthSubscription.unsubscribe();
    }
}
PlDatepickerOverlayComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-datepicker-overlay',
                template: "<section class=\"pl-datepicker-overlay\">\n    <pl-datepicker-header></pl-datepicker-header>\n    <pl-datepicker-calendar></pl-datepicker-calendar>\n</section>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
PlDatepickerOverlayComponent.ctorParameters = () => [
    { type: PlDatepickerService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlDatepickerCalendarComponent {
    /**
     * @param {?} plDatepickerService
     */
    constructor(plDatepickerService) {
        this.plDatepickerService = plDatepickerService;
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        date => this.currentMonth = date));
    }
    /**
     * @return {?}
     */
    getDaysInMonth() {
        return this.plDatepickerService.getMonthMatrix(this.currentMonth);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isInCurrentMonth(date) {
        return this.plDatepickerService.isInCurrentMonth(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isSelectedDate(date) {
        return this.plDatepickerService.isSelectedDate(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    isToday(date) {
        return this.plDatepickerService.isToday(date);
    }
    /**
     * @param {?} date
     * @return {?}
     */
    onDateSelect(date) {
        this.plDatepickerService.setSelectedDate(date);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.currentMonthSubscription.unsubscribe();
    }
}
PlDatepickerCalendarComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-datepicker-calendar',
                template: "<div class=\"pl-datepicker-calendar\">\n    <ul class=\"pl-datepicker-calendar__weeks\">\n        <li class=\"pl-datepicker-calendar__week-header\">\n            <ul class=\"pl-datepicker-calendar__day-labels\">\n                <li class=\"pl-datepicker-calendar__day-label\" *ngFor=\"let day of getDaysInMonth()[0]['days'];\">{{ day | date:'EEEEE' }}</li>\n            </ul>\n        </li>\n\n        <li class=\"pl-datepicker-calendar__week\"\n            *ngFor=\"let week of getDaysInMonth();\">\n                <ul class=\"pl-datepicker-calendar__days\">\n                    <li class=\"pl-datepicker-calendar__week-no\">{{ week.no }}</li>\n                    <li \n                        class=\"pl-datepicker-calendar__day\"\n                        [ngClass]=\"{\n                            'pl-datepicker-calendar__day--selected': isSelectedDate(day),\n                            'pl-datepicker-calendar__day--today': isToday(day),\n                            'pl-datepicker-calendar__day--current-month': isInCurrentMonth(day)\n                        }\"\n                        *ngFor=\"let day of week.days;\"\n                        (click)=\"onDateSelect(day)\">\n                            {{ day | date:'dd' }}\n                    </li>\n                </ul>\n        </li>\n\n    </ul>\n</div>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
PlDatepickerCalendarComponent.ctorParameters = () => [
    { type: PlDatepickerService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlDatepickerHeaderComponent {
    /**
     * @param {?} plDatepickerService
     */
    constructor(plDatepickerService) {
        this.plDatepickerService = plDatepickerService;
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        date => this.currentMonth = date));
    }
    /**
     * @return {?}
     */
    onNavigateToNextMonth() {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() + 1, 1));
    }
    /**
     * @return {?}
     */
    onNavigateToPreviousMonth() {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() - 1, 1));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.currentMonthSubscription.unsubscribe();
    }
}
PlDatepickerHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-datepicker-header',
                template: "<header class=\"pl-datepicker-header\">\n    <button class=\"pl-datepicker-header__btn-next-month\" (click)=\"onNavigateToPreviousMonth()\"><</button>\n    <span class=\"pl-datepicker-header__current-month\">{{ currentMonth | date:'MMMM yyyy' }}</span>\n    <button class=\"pl-datepicker-header__btn-prev-month\" (click)=\"onNavigateToNextMonth()\">></button>\n</header>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
PlDatepickerHeaderComponent.ctorParameters = () => [
    { type: PlDatepickerService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PlInputComponent {
    constructor() {
        this.showValidationMessage = false;
        this.propagateChange = (/**
         * @param {?} value
         * @return {?}
         */
        (value) => value);
        this.propagateTouch = noop;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onChange(value) {
        this.propagateChange(value.data);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this.model = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.propagateTouch = fn;
    }
}
PlInputComponent.decorators = [
    { type: Component, args: [{
                selector: 'pl-input',
                template: "<div class=\"pl-input\">\n    <label class=\"pl-input__label\">{{ label }}</label>\n    <input [type]=\"type\" class=\"pl-input__control\" (input)=\"onChange($event)\" [ngModel]=\"model\">\n    <div class=\"pl-input__addition\">\n        <div class=\"pl-input__help-text\" *ngIf=\"!showValidationMessage\">{{ helpText }}</div>\n        <div class=\"pl-input__validation-msg\" *ngIf=\"showValidationMessage\">{{ validationMessage }}</div>\n    </div>\n</div>",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => PlInputComponent)),
                        multi: true,
                    }
                ],
                styles: [""]
            }] }
];
/** @nocollapse */
PlInputComponent.ctorParameters = () => [];
PlInputComponent.propDecorators = {
    label: [{ type: Input }],
    type: [{ type: Input }],
    helpText: [{ type: Input }],
    validationMessage: [{ type: Input }],
    showValidationMessage: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgParallacticUiModule {
}
NgParallacticUiModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ClickOutsideDirective,
                    PlTableComponent,
                    PlTableRowComponent,
                    PlTableCellComponent,
                    PlTableCellDirective,
                    PlTableCellRendererComponent,
                    PlTableHeaderComponent,
                    PlTableHeaderCellComponent,
                    PlDatepickerComponent,
                    PlDatepickerOverlayComponent,
                    PlDatepickerCalendarComponent,
                    PlDatepickerHeaderComponent,
                    PlInputComponent,
                ],
                providers: [
                    ClickOutsideDirective
                ],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [
                    PlTableComponent,
                    PlDatepickerComponent,
                    PlInputComponent,
                ],
                entryComponents: [
                    PlTableCellRendererComponent
                ]
            },] }
];

export { NgParallacticUiModule, ClickOutsideDirective as ɵa, PlTableComponent as ɵb, PlTableCellService as ɵc, PlTableRowComponent as ɵd, PlTableCellComponent as ɵe, PlTableCellDirective as ɵf, PlTableCellRendererComponent as ɵg, PlTableHeaderComponent as ɵh, PlTableHeaderCellComponent as ɵi, PlDatepickerComponent as ɵj, PlDatepickerService as ɵk, PlDatepickerOverlayComponent as ɵl, PlDatepickerCalendarComponent as ɵm, PlDatepickerHeaderComponent as ɵn, PlInputComponent as ɵo };
//# sourceMappingURL=ng-parallactic-ui.js.map
