import { Directive, ElementRef, Output, HostListener, EventEmitter, Injectable, ɵɵdefineInjectable, Component, Input, ViewContainerRef, ComponentFactoryResolver, ViewChild, forwardRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { BehaviorSubject, noop } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ClickOutsideDirective = /** @class */ (function () {
    function ClickOutsideDirective(_elementRef) {
        this._elementRef = _elementRef;
        this.appClickOutside = new EventEmitter();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    ClickOutsideDirective.prototype.onClick = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var _this = this;
        /** @type {?} */
        var targetElementPath = typeof event.composedPath === 'function' ? event.composedPath() : this._getComposedPath(event.target);
        /** @type {?} */
        var elementRefInPath = targetElementPath.find((/**
         * @param {?} e
         * @return {?}
         */
        function (e) { return e === _this._elementRef.nativeElement; }));
        if (!elementRefInPath) {
            this.appClickOutside.emit(event);
        }
    };
    /**
     * @private
     * @param {?} node
     * @return {?}
     */
    ClickOutsideDirective.prototype._getComposedPath = /**
     * @private
     * @param {?} node
     * @return {?}
     */
    function (node) {
        /** @type {?} */
        var parent;
        if (node.parentNode) {
            parent = node.parentNode;
        }
        else if (node.host) {
            parent = node.host;
        }
        else if (node.defaultView) {
            parent = node.defaultView;
        }
        if (parent !== undefined) {
            return [node].concat(this._getComposedPath(parent));
        }
        return [node];
    };
    ClickOutsideDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[clickOutside]'
                },] }
    ];
    /** @nocollapse */
    ClickOutsideDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    ClickOutsideDirective.propDecorators = {
        appClickOutside: [{ type: Output }],
        onClick: [{ type: HostListener, args: ['document:click', ['$event'],] }]
    };
    return ClickOutsideDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableCellService = /** @class */ (function () {
    function PlTableCellService() {
    }
    /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    PlTableCellService.prototype.getTableCellValue = /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    function (tableRowData, tableColumnDefinition) {
        if (!tableRowData || !tableColumnDefinition)
            return;
        /** @type {?} */
        var fields = tableColumnDefinition.field.split('.');
        /** @type {?} */
        var cellValue = tableRowData;
        for (var i = 0; i < fields.length; i++) {
            /** @type {?} */
            var key = fields[i];
            if (cellValue[key])
                cellValue = cellValue[key];
            else
                return undefined;
        }
        if (tableColumnDefinition.valueFormatter) {
            cellValue = tableColumnDefinition.valueFormatter(cellValue, tableRowData, tableColumnDefinition);
        }
        return { value: cellValue };
    };
    PlTableCellService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    PlTableCellService.ctorParameters = function () { return []; };
    /** @nocollapse */ PlTableCellService.ngInjectableDef = ɵɵdefineInjectable({ factory: function PlTableCellService_Factory() { return new PlTableCellService(); }, token: PlTableCellService, providedIn: "root" });
    return PlTableCellService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableComponent = /** @class */ (function () {
    function PlTableComponent(plTableCellService) {
        this.plTableCellService = plTableCellService;
        this._tableData = [];
        this.appliedTableOptions = {};
        this.selectedTableRows = [];
        this.tableOptions = {};
        this.tableRowClicked = new EventEmitter();
        this.tableRowSelectionChanged = new EventEmitter();
    }
    Object.defineProperty(PlTableComponent.prototype, "tableData", {
        get: /**
         * @return {?}
         */
        function () {
            return this._tableData;
        },
        set: /**
         * @param {?} tableData
         * @return {?}
         */
        function (tableData) {
            if (this.tableColumnsDefinitions) {
                this._tableData = tableData ? tableData : [];
                this.applyTableOptions(this.tableOptions);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    PlTableComponent.prototype.applyTableOptions = /**
     * @private
     * @param {?} options
     * @return {?}
     */
    function (options) {
        this.appliedTableOptions.sortBy = options.sortBy || null;
        this.appliedTableOptions.sortOrder = options.sortOrder || 'asc';
        this.appliedTableOptions.selectableRows = options.selectableRows || false;
        this.appliedTableOptions.selectOnRowClick = options.selectOnRowClick || false;
        this.sortTableData();
    };
    /* Sort table rows */
    /* Sort table rows */
    /**
     * @param {?} sortOptions
     * @return {?}
     */
    PlTableComponent.prototype.onSortChange = /* Sort table rows */
    /**
     * @param {?} sortOptions
     * @return {?}
     */
    function (sortOptions) {
        if (!sortOptions)
            return;
        this.appliedTableOptions.sortBy = sortOptions.tableColumnKey;
        this.appliedTableOptions.sortOrder = sortOptions.sortOrder;
        this.sortTableData();
    };
    /**
     * @private
     * @return {?}
     */
    PlTableComponent.prototype.sortTableData = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        this.tableData.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        function (a, b) {
            /** @type {?} */
            var orderModifyer = _this.appliedTableOptions.sortOrder === 'asc' ? 1 : -1;
            /** @type {?} */
            var tableColumnDefinition = _this.tableColumnsDefinitions.find((/**
             * @param {?} item
             * @return {?}
             */
            function (item) { return item.field === _this.appliedTableOptions.sortBy; }));
            /** @type {?} */
            var tableCellA = _this.plTableCellService.getTableCellValue(a, tableColumnDefinition);
            /** @type {?} */
            var tableCellB = _this.plTableCellService.getTableCellValue(b, tableColumnDefinition);
            /** @type {?} */
            var tableCellAValue = tableCellA ? tableCellA.value : 0;
            /** @type {?} */
            var tableCellBValue = tableCellB ? tableCellB.value : 0;
            if (typeof tableCellAValue === 'string') {
                tableCellAValue = tableCellAValue.toLowerCase();
            }
            if (typeof tableCellBValue === 'string') {
                tableCellBValue = tableCellBValue.toLowerCase();
            }
            if (tableCellAValue > tableCellBValue)
                return 1 * orderModifyer;
            if (tableCellAValue < tableCellBValue)
                return -1 * orderModifyer;
            return 0;
        }));
    };
    /* Selected table rows */
    /* Selected table rows */
    /**
     * @param {?} tableRow
     * @return {?}
     */
    PlTableComponent.prototype.onSelectedTableRowsToggle = /* Selected table rows */
    /**
     * @param {?} tableRow
     * @return {?}
     */
    function (tableRow) {
        if (this.selectedTableRows.indexOf(tableRow) > -1) {
            this.onSelectedTableRowsRemove(tableRow);
        }
        else {
            this.onSelectedTableRowsAdd(tableRow);
        }
    };
    /**
     * @param {?} tableRow
     * @return {?}
     */
    PlTableComponent.prototype.onSelectedTableRowsAdd = /**
     * @param {?} tableRow
     * @return {?}
     */
    function (tableRow) {
        this.selectedTableRows.push(tableRow);
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    };
    /**
     * @param {?} tableRow
     * @return {?}
     */
    PlTableComponent.prototype.onSelectedTableRowsRemove = /**
     * @param {?} tableRow
     * @return {?}
     */
    function (tableRow) {
        /** @type {?} */
        var rowIndex = this.selectedTableRows.indexOf(tableRow);
        if (rowIndex > -1) {
            this.selectedTableRows.splice(rowIndex, 1);
            this.tableRowSelectionChanged.emit(this.selectedTableRows);
        }
    };
    /**
     * @return {?}
     */
    PlTableComponent.prototype.onSelectedTableRowsAddAll = /**
     * @return {?}
     */
    function () {
        this.selectedTableRows = this.tableData;
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    };
    /**
     * @return {?}
     */
    PlTableComponent.prototype.onSelectedTableRowsRemoveAll = /**
     * @return {?}
     */
    function () {
        this.selectedTableRows = [];
        this.tableRowSelectionChanged.emit(this.selectedTableRows);
    };
    /* Table events */
    /* Table events */
    /**
     * @param {?} event
     * @param {?} tableRowData
     * @param {?} tableRowIndex
     * @return {?}
     */
    PlTableComponent.prototype.onTableRowClick = /* Table events */
    /**
     * @param {?} event
     * @param {?} tableRowData
     * @param {?} tableRowIndex
     * @return {?}
     */
    function (event, tableRowData, tableRowIndex) {
        this.tableRowClicked.emit({ event: event, tableRowData: tableRowData, tableRowIndex: tableRowIndex });
        // select row if "selectOnRowClick" option is set true
        if (this.appliedTableOptions.selectableRows
            && this.appliedTableOptions.selectOnRowClick) {
            this.onSelectedTableRowsToggle(tableRowData);
        }
    };
    PlTableComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table',
                    template: "<pl-table-header\n    class=\"pl-table__header\"\n    [tableOptions]=\"appliedTableOptions\"\n    [tableColumnDefinitions]=\"tableColumnsDefinitions\"\n    [allTabelRowsAreSelected]=\"tableData.length === selectedTableRows.length\"\n    (sortChange)=\"onSortChange($event)\"\n    (allRowsSelected)=\"onSelectedTableRowsAddAll()\"\n    (AllRowsUnselected)=\"onSelectedTableRowsRemoveAll()\">\n</pl-table-header>\n<pl-table-row \n    class=\"pl-table__row\"\n    *ngFor=\"let tableRowData of tableData; let i = index\"\n    [tableOptions]=\"appliedTableOptions\"\n    [tableColumnDefinitions]=\"tableColumnsDefinitions\"\n    [tableRowData]=\"tableRowData\"\n    [selectedTableRows]=\"selectedTableRows\"\n    (rowSelected)=\"onSelectedTableRowsAdd($event)\"\n    (rowUnselected)=\"onSelectedTableRowsRemove($event)\"\n    (click)=\"onTableRowClick($event, tableRowData, i)\">\n</pl-table-row>",
                    host: { '[class.pl-table]': 'true' },
                    providers: [PlTableCellService],
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlTableComponent.ctorParameters = function () { return [
        { type: PlTableCellService }
    ]; };
    PlTableComponent.propDecorators = {
        tableOptions: [{ type: Input }],
        tableColumnsDefinitions: [{ type: Input }],
        tableRowClicked: [{ type: Output }],
        tableRowSelectionChanged: [{ type: Output }],
        tableData: [{ type: Input, args: ['tableData',] }]
    };
    return PlTableComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableRowComponent = /** @class */ (function () {
    function PlTableRowComponent(plTableCellService) {
        this.plTableCellService = plTableCellService;
        this.rowSelected = new EventEmitter();
        this.rowUnselected = new EventEmitter();
    }
    /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    PlTableRowComponent.prototype.getCellValue = /**
     * @param {?} tableRowData
     * @param {?} tableColumnDefinition
     * @return {?}
     */
    function (tableRowData, tableColumnDefinition) {
        return this.plTableCellService.getTableCellValue(tableRowData, tableColumnDefinition);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    PlTableRowComponent.prototype.onTableRowSelect = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        event.stopPropagation();
        if (this.isSelectedTableRow())
            this.rowUnselected.emit(this.tableRowData);
        else
            this.rowSelected.emit(this.tableRowData);
    };
    /**
     * @return {?}
     */
    PlTableRowComponent.prototype.isSelectedTableRow = /**
     * @return {?}
     */
    function () {
        return this.selectedTableRows.indexOf(this.tableRowData) > -1;
    };
    PlTableRowComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table-row',
                    template: "<div class=\"pl-table__row__selector\" *ngIf=\"tableOptions.selectableRows\">\n    <input type=\"checkbox\" (click)=\"onTableRowSelect($event)\" [checked]=\"isSelectedTableRow()\">\n</div>\n<pl-table-cell \n    class=\"pl-table__cell\"\n    *ngFor=\"let tableColumnDefinition of tableColumnDefinitions; let i = index\"\n    [tableCellData]=\"getCellValue(tableRowData, tableColumnDefinition)\"\n    [tableColumnDefinition]=\"tableColumnDefinition\">\n</pl-table-cell>",
                    styles: [":host{display:-webkit-box;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;flex-direction:row;flex-wrap:nowrap;width:100%}"]
                }] }
    ];
    /** @nocollapse */
    PlTableRowComponent.ctorParameters = function () { return [
        { type: PlTableCellService }
    ]; };
    PlTableRowComponent.propDecorators = {
        tableOptions: [{ type: Input }],
        tableColumnDefinitions: [{ type: Input }],
        tableRowData: [{ type: Input }],
        selectedTableRows: [{ type: Input }],
        rowSelected: [{ type: Output }],
        rowUnselected: [{ type: Output }]
    };
    return PlTableRowComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableCellDirective = /** @class */ (function () {
    function PlTableCellDirective(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
    PlTableCellDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[pl-table-cell]'
                },] }
    ];
    /** @nocollapse */
    PlTableCellDirective.ctorParameters = function () { return [
        { type: ViewContainerRef }
    ]; };
    return PlTableCellDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableCellRendererComponent = /** @class */ (function () {
    function PlTableCellRendererComponent() {
    }
    PlTableCellRendererComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table-cell-renderer',
                    template: "{{ tableCellData?.value }}",
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlTableCellRendererComponent.ctorParameters = function () { return []; };
    PlTableCellRendererComponent.propDecorators = {
        tableCellData: [{ type: Input }]
    };
    return PlTableCellRendererComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableCellComponent = /** @class */ (function () {
    function PlTableCellComponent(componentFactoryResolver) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.defaultTableCellRenderer = PlTableCellRendererComponent;
    }
    /**
     * @return {?}
     */
    PlTableCellComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.loadTableCellRenderer();
    };
    /**
     * @private
     * @return {?}
     */
    PlTableCellComponent.prototype.loadTableCellRenderer = /**
     * @private
     * @return {?}
     */
    function () {
        /** @type {?} */
        var tableCellRenderer = this.tableColumnDefinition.tableCellRenderer || this.defaultTableCellRenderer;
        /** @type {?} */
        var componentFactory = this.componentFactoryResolver.resolveComponentFactory(tableCellRenderer);
        /** @type {?} */
        var viewContainerRef = this.tableCellRenderer.viewContainerRef;
        viewContainerRef.clear();
        /** @type {?} */
        var componentRef = viewContainerRef.createComponent(componentFactory);
        ((/** @type {?} */ (componentRef.instance))).tableCellData = this.tableCellData;
    };
    PlTableCellComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table-cell',
                    template: "<ng-template pl-table-cell></ng-template>",
                    styles: [":host{-webkit-box-flex:1;flex-grow:1;width:20rem}"]
                }] }
    ];
    /** @nocollapse */
    PlTableCellComponent.ctorParameters = function () { return [
        { type: ComponentFactoryResolver }
    ]; };
    PlTableCellComponent.propDecorators = {
        tableCellData: [{ type: Input }],
        tableColumnDefinition: [{ type: Input }],
        tableCellRenderer: [{ type: ViewChild, args: [PlTableCellDirective, { static: true },] }]
    };
    return PlTableCellComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableHeaderComponent = /** @class */ (function () {
    function PlTableHeaderComponent() {
        this.sortChange = new EventEmitter();
        this.allRowsSelected = new EventEmitter();
        this.AllRowsUnselected = new EventEmitter();
    }
    /**
     * @param {?} eventData
     * @return {?}
     */
    PlTableHeaderComponent.prototype.onSortChange = /**
     * @param {?} eventData
     * @return {?}
     */
    function (eventData) {
        this.sortChange.emit(eventData);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    PlTableHeaderComponent.prototype.onTableRowSelectAll = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        event.stopPropagation();
        if (!this.allTabelRowsAreSelected)
            this.allRowsSelected.emit();
        else
            this.AllRowsUnselected.emit();
    };
    PlTableHeaderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table-header',
                    template: "<div class=\"pl-table__header__row-selector\" *ngIf=\"tableOptions.selectableRows\">\n    <input type=\"checkbox\" (click)=\"onTableRowSelectAll($event)\" [checked]=\"allTabelRowsAreSelected\">\n</div>\n<pl-table-header-cell \n    class=\"pl-table__header__cell \"\n    *ngFor=\"let tableColumnDefinition of tableColumnDefinitions; let i = index\"\n    [tableOptions]=\"tableOptions\"\n    [tableColumnDefinition]=\"tableColumnDefinition\"\n    (sortChange)=\"onSortChange($event)\">\n</pl-table-header-cell>",
                    styles: [":host{display:-webkit-box;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;flex-direction:row;flex-wrap:nowrap;width:100%}"]
                }] }
    ];
    /** @nocollapse */
    PlTableHeaderComponent.ctorParameters = function () { return []; };
    PlTableHeaderComponent.propDecorators = {
        tableOptions: [{ type: Input }],
        tableColumnDefinitions: [{ type: Input }],
        allTabelRowsAreSelected: [{ type: Input }],
        sortChange: [{ type: Output }],
        allRowsSelected: [{ type: Output }],
        AllRowsUnselected: [{ type: Output }]
    };
    return PlTableHeaderComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlTableHeaderCellComponent = /** @class */ (function () {
    function PlTableHeaderCellComponent() {
        this.sortChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    PlTableHeaderCellComponent.prototype.onLabelClick = /**
     * @return {?}
     */
    function () {
        if (this.tableColumnDefinition.sortable) {
            /** @type {?} */
            var sortOrder = this.tableOptions.sortBy === this.tableColumnDefinition.field ? this.tableOptions.sortOrder === 'asc' ? 'desc' : 'asc' : 'asc';
            this.sortChange.emit({ tableColumnKey: this.tableColumnDefinition.field, sortOrder: sortOrder });
        }
    };
    PlTableHeaderCellComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-table-header-cell',
                    template: "<span \n    class=\"pl-table__header__cell__label\"\n    [ngClass]=\"{\n        'pl-table__header__cell__label--sortable': tableColumnDefinition.sortable,\n        'pl-table__header__cell__label--sorted': tableColumnDefinition.field === tableOptions.sortBy,\n        'pl-table__header__cell__label--sorted-asc': tableColumnDefinition.field === tableOptions.sortBy && tableOptions.sortOrder === 'asc',\n        'pl-table__header__cell__label--sorted-desc': tableColumnDefinition.field === tableOptions.sortBy && tableOptions.sortOrder === 'desc'\n    }\"\n    (click)=\"onLabelClick()\">\n        {{ tableColumnDefinition.title }}\n</span>",
                    styles: [":host{-webkit-box-flex:1;flex-grow:1;width:20rem}"]
                }] }
    ];
    /** @nocollapse */
    PlTableHeaderCellComponent.ctorParameters = function () { return []; };
    PlTableHeaderCellComponent.propDecorators = {
        tableOptions: [{ type: Input }],
        tableColumnDefinition: [{ type: Input }],
        sortChange: [{ type: Output }]
    };
    return PlTableHeaderCellComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlDatepickerService = /** @class */ (function () {
    function PlDatepickerService() {
        this.selectedDate$ = new BehaviorSubject(this.selectedDate);
        this.currentMonth$ = new BehaviorSubject(this.currentMonth);
        this.setCurrentMonth(this.getFirstDayOfMonth(new Date()));
    }
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.setSelectedDate = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        this.selectedDate = date;
        this.selectedDate$.next(this.selectedDate);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.setCurrentMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        this.currentMonth = this.getFirstDayOfMonth(date);
        this.currentMonth$.next(this.currentMonth);
    };
    /**
     * @param {?} month
     * @return {?}
     */
    PlDatepickerService.prototype.getMonthMatrix = /**
     * @param {?} month
     * @return {?}
     */
    function (month) {
        if (!this.isTypeOfDate(month))
            return;
        /** @type {?} */
        var firstDayWeekIndex = this.adjustWeekIndex(this.getFirstDayOfMonth(month).getDay());
        /** @type {?} */
        var matrixStartDate = new Date(month.getFullYear(), month.getMonth(), 1 - firstDayWeekIndex);
        /** @type {?} */
        var daysOfTheMonth = [];
        /** @type {?} */
        var days = [];
        for (var i = 0; i <= 42; i++) {
            /** @type {?} */
            var date = new Date(matrixStartDate.getFullYear(), matrixStartDate.getMonth(), matrixStartDate.getDate() + i);
            days.push(date);
            if ((i + 1) % 7 === 0) {
                daysOfTheMonth.push({
                    no: this.getWeek(date),
                    days: days
                });
                days = [];
            }
        }
        return daysOfTheMonth;
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.isTypeOfDate = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return Object.prototype.toString.call(date) === '[object Date]';
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.getFirstDayOfMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        if (!this.isTypeOfDate(date)) {
            date = new Date();
        }
        return new Date(date.getFullYear(), date.getMonth(), 1);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.getLastDayOfMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        if (!this.isTypeOfDate(date)) {
            date = new Date();
        }
        return new Date(date.getFullYear(), date.getMonth() + 1, 0);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.isInCurrentMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return date.getMonth() === this.currentMonth.getMonth();
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.isSelectedDate = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.isSameDay(date, this.selectedDate);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerService.prototype.isToday = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.isSameDay(date, new Date());
    };
    /**
     * @param {?} date1
     * @param {?} date2
     * @return {?}
     */
    PlDatepickerService.prototype.isSameDay = /**
     * @param {?} date1
     * @param {?} date2
     * @return {?}
     */
    function (date1, date2) {
        if (!this.isTypeOfDate(date1) || !this.isTypeOfDate(date2))
            return;
        return date1.getDate() == date2.getDate() &&
            date1.getMonth() == date2.getMonth() &&
            date1.getFullYear() == date2.getFullYear();
    };
    /**
     * @param {?} index
     * @return {?}
     */
    PlDatepickerService.prototype.adjustWeekIndex = /**
     * @param {?} index
     * @return {?}
     */
    function (index) {
        /** @type {?} */
        var offset = 1;
        return index < offset ? 7 - offset : index - offset;
    };
    /**
     * @param {?} _date
     * @return {?}
     */
    PlDatepickerService.prototype.getWeek = /**
     * @param {?} _date
     * @return {?}
     */
    function (_date) {
        if (!this.isTypeOfDate(_date))
            return 0;
        /** @type {?} */
        var date = new Date(_date.getTime());
        date.setHours(0, 0, 0, 0);
        date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
        /** @type {?} */
        var week1 = new Date(date.getFullYear(), 0, 4);
        return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000 - 3 + (week1.getDay() + 6) % 7) / 7);
    };
    PlDatepickerService.decorators = [
        { type: Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    /** @nocollapse */
    PlDatepickerService.ctorParameters = function () { return []; };
    /** @nocollapse */ PlDatepickerService.ngInjectableDef = ɵɵdefineInjectable({ factory: function PlDatepickerService_Factory() { return new PlDatepickerService(); }, token: PlDatepickerService, providedIn: "root" });
    return PlDatepickerService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlDatepickerComponent = /** @class */ (function () {
    function PlDatepickerComponent(plDatepickerService) {
        var _this = this;
        this.plDatepickerService = plDatepickerService;
        this.showOverlay = false;
        this.appliedOptions = {};
        this.datepickerOptions = {};
        this.propagateChange = (/**
         * @param {?} date
         * @return {?}
         */
        function (date) { return date; });
        this.propagateTouch = noop;
        this.selectedDateSubscription = this.plDatepickerService.selectedDate$.subscribe((/**
         * @param {?} date
         * @return {?}
         */
        function (date) {
            _this.onDateSelect(date);
        }));
        this.applyOptions(this.datepickerOptions);
    }
    /**
     * @return {?}
     */
    PlDatepickerComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.applyOptions(this.datepickerOptions);
    };
    /**
     * @private
     * @param {?} options
     * @return {?}
     */
    PlDatepickerComponent.prototype.applyOptions = /**
     * @private
     * @param {?} options
     * @return {?}
     */
    function (options) {
        this.appliedOptions.dateFormat = options.dateFormat || 'dd.MM.yyyy';
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerComponent.prototype.onDateSelect = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        this.model = date;
        this.propagateChange(this.model);
        if (this.model) {
            this.showOverlay = false;
        }
    };
    /**
     * @return {?}
     */
    PlDatepickerComponent.prototype.toggleDatepickerOverlay = /**
     * @return {?}
     */
    function () {
        this.showOverlay = !this.showOverlay;
        this.propagateTouch();
    };
    /**
     * @return {?}
     */
    PlDatepickerComponent.prototype.hideOverlay = /**
     * @return {?}
     */
    function () {
        this.showOverlay = false;
    };
    /**
     * @param {?} obj
     * @return {?}
     */
    PlDatepickerComponent.prototype.writeValue = /**
     * @param {?} obj
     * @return {?}
     */
    function (obj) {
        this.plDatepickerService.setSelectedDate(obj || null);
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    PlDatepickerComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.propagateChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    PlDatepickerComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.propagateTouch = fn;
    };
    /**
     * @return {?}
     */
    PlDatepickerComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.selectedDateSubscription.unsubscribe();
    };
    PlDatepickerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-datepicker',
                    template: "<div class=\"pl-datepicker\" (clickOutside)=\"hideOverlay()\">\n    <span class=\"pl-datepicker__model\" (click)=\"toggleDatepickerOverlay()\">{{ model | date:appliedOptions.dateFormat }}</span>\n    <pl-datepicker-overlay *ngIf=\"showOverlay\"></pl-datepicker-overlay>\n</div>",
                    providers: [
                        PlDatepickerService,
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef((/**
                             * @return {?}
                             */
                            function () { return PlDatepickerComponent; })),
                            multi: true,
                        }
                    ],
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlDatepickerComponent.ctorParameters = function () { return [
        { type: PlDatepickerService }
    ]; };
    PlDatepickerComponent.propDecorators = {
        datepickerOptions: [{ type: Input }]
    };
    return PlDatepickerComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlDatepickerOverlayComponent = /** @class */ (function () {
    function PlDatepickerOverlayComponent(plDatepickerService) {
        var _this = this;
        this.plDatepickerService = plDatepickerService;
        this.selectedDateSubscription = this.plDatepickerService.selectedDate$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        function (date) { return _this.model = date; }));
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        function (date) { return _this.currentMonth = date; }));
    }
    /**
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.getDaysInMonth = /**
     * @return {?}
     */
    function () {
        return this.plDatepickerService.getMonthMatrix(this.currentMonth);
    };
    /**
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.onNavigateToNextMonth = /**
     * @return {?}
     */
    function () {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() + 1, 1));
    };
    /**
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.onNavigateToPreviousMonth = /**
     * @return {?}
     */
    function () {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() - 1, 1));
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.isInCurrentMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isInCurrentMonth(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.isSelectedDate = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isSelectedDate(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.isToday = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isToday(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.onDateSelect = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        this.plDatepickerService.setSelectedDate(date);
    };
    /**
     * @return {?}
     */
    PlDatepickerOverlayComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.selectedDateSubscription.unsubscribe();
        this.currentMonthSubscription.unsubscribe();
    };
    PlDatepickerOverlayComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-datepicker-overlay',
                    template: "<section class=\"pl-datepicker-overlay\">\n    <pl-datepicker-header></pl-datepicker-header>\n    <pl-datepicker-calendar></pl-datepicker-calendar>\n</section>\n",
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlDatepickerOverlayComponent.ctorParameters = function () { return [
        { type: PlDatepickerService }
    ]; };
    return PlDatepickerOverlayComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlDatepickerCalendarComponent = /** @class */ (function () {
    function PlDatepickerCalendarComponent(plDatepickerService) {
        var _this = this;
        this.plDatepickerService = plDatepickerService;
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        function (date) { return _this.currentMonth = date; }));
    }
    /**
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.getDaysInMonth = /**
     * @return {?}
     */
    function () {
        return this.plDatepickerService.getMonthMatrix(this.currentMonth);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.isInCurrentMonth = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isInCurrentMonth(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.isSelectedDate = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isSelectedDate(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.isToday = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        return this.plDatepickerService.isToday(date);
    };
    /**
     * @param {?} date
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.onDateSelect = /**
     * @param {?} date
     * @return {?}
     */
    function (date) {
        this.plDatepickerService.setSelectedDate(date);
    };
    /**
     * @return {?}
     */
    PlDatepickerCalendarComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.currentMonthSubscription.unsubscribe();
    };
    PlDatepickerCalendarComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-datepicker-calendar',
                    template: "<div class=\"pl-datepicker-calendar\">\n    <ul class=\"pl-datepicker-calendar__weeks\">\n        <li class=\"pl-datepicker-calendar__week-header\">\n            <ul class=\"pl-datepicker-calendar__day-labels\">\n                <li class=\"pl-datepicker-calendar__day-label\" *ngFor=\"let day of getDaysInMonth()[0]['days'];\">{{ day | date:'EEEEE' }}</li>\n            </ul>\n        </li>\n\n        <li class=\"pl-datepicker-calendar__week\"\n            *ngFor=\"let week of getDaysInMonth();\">\n                <ul class=\"pl-datepicker-calendar__days\">\n                    <li class=\"pl-datepicker-calendar__week-no\">{{ week.no }}</li>\n                    <li \n                        class=\"pl-datepicker-calendar__day\"\n                        [ngClass]=\"{\n                            'pl-datepicker-calendar__day--selected': isSelectedDate(day),\n                            'pl-datepicker-calendar__day--today': isToday(day),\n                            'pl-datepicker-calendar__day--current-month': isInCurrentMonth(day)\n                        }\"\n                        *ngFor=\"let day of week.days;\"\n                        (click)=\"onDateSelect(day)\">\n                            {{ day | date:'dd' }}\n                    </li>\n                </ul>\n        </li>\n\n    </ul>\n</div>\n",
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlDatepickerCalendarComponent.ctorParameters = function () { return [
        { type: PlDatepickerService }
    ]; };
    return PlDatepickerCalendarComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlDatepickerHeaderComponent = /** @class */ (function () {
    function PlDatepickerHeaderComponent(plDatepickerService) {
        var _this = this;
        this.plDatepickerService = plDatepickerService;
        this.currentMonthSubscription = this.plDatepickerService.currentMonth$
            .subscribe((/**
         * @param {?} date
         * @return {?}
         */
        function (date) { return _this.currentMonth = date; }));
    }
    /**
     * @return {?}
     */
    PlDatepickerHeaderComponent.prototype.onNavigateToNextMonth = /**
     * @return {?}
     */
    function () {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() + 1, 1));
    };
    /**
     * @return {?}
     */
    PlDatepickerHeaderComponent.prototype.onNavigateToPreviousMonth = /**
     * @return {?}
     */
    function () {
        this.plDatepickerService.setCurrentMonth(new Date(this.currentMonth.getFullYear(), this.currentMonth.getMonth() - 1, 1));
    };
    /**
     * @return {?}
     */
    PlDatepickerHeaderComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this.currentMonthSubscription.unsubscribe();
    };
    PlDatepickerHeaderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-datepicker-header',
                    template: "<header class=\"pl-datepicker-header\">\n    <button class=\"pl-datepicker-header__btn-next-month\" (click)=\"onNavigateToPreviousMonth()\"><</button>\n    <span class=\"pl-datepicker-header__current-month\">{{ currentMonth | date:'MMMM yyyy' }}</span>\n    <button class=\"pl-datepicker-header__btn-prev-month\" (click)=\"onNavigateToNextMonth()\">></button>\n</header>\n",
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlDatepickerHeaderComponent.ctorParameters = function () { return [
        { type: PlDatepickerService }
    ]; };
    return PlDatepickerHeaderComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var PlInputComponent = /** @class */ (function () {
    function PlInputComponent() {
        this.showValidationMessage = false;
        this.propagateChange = (/**
         * @param {?} value
         * @return {?}
         */
        function (value) { return value; });
        this.propagateTouch = noop;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    PlInputComponent.prototype.onChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.propagateChange(value.data);
    };
    /**
     * @param {?} value
     * @return {?}
     */
    PlInputComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.model = value;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    PlInputComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.propagateChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    PlInputComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.propagateTouch = fn;
    };
    PlInputComponent.decorators = [
        { type: Component, args: [{
                    selector: 'pl-input',
                    template: "<div class=\"pl-input\">\n    <label class=\"pl-input__label\">{{ label }}</label>\n    <input [type]=\"type\" class=\"pl-input__control\" (input)=\"onChange($event)\" [ngModel]=\"model\">\n    <div class=\"pl-input__addition\">\n        <div class=\"pl-input__help-text\" *ngIf=\"!showValidationMessage\">{{ helpText }}</div>\n        <div class=\"pl-input__validation-msg\" *ngIf=\"showValidationMessage\">{{ validationMessage }}</div>\n    </div>\n</div>",
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef((/**
                             * @return {?}
                             */
                            function () { return PlInputComponent; })),
                            multi: true,
                        }
                    ],
                    styles: [""]
                }] }
    ];
    /** @nocollapse */
    PlInputComponent.ctorParameters = function () { return []; };
    PlInputComponent.propDecorators = {
        label: [{ type: Input }],
        type: [{ type: Input }],
        helpText: [{ type: Input }],
        validationMessage: [{ type: Input }],
        showValidationMessage: [{ type: Input }]
    };
    return PlInputComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NgParallacticUiModule = /** @class */ (function () {
    function NgParallacticUiModule() {
    }
    NgParallacticUiModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [
                        ClickOutsideDirective,
                        PlTableComponent,
                        PlTableRowComponent,
                        PlTableCellComponent,
                        PlTableCellDirective,
                        PlTableCellRendererComponent,
                        PlTableHeaderComponent,
                        PlTableHeaderCellComponent,
                        PlDatepickerComponent,
                        PlDatepickerOverlayComponent,
                        PlDatepickerCalendarComponent,
                        PlDatepickerHeaderComponent,
                        PlInputComponent,
                    ],
                    providers: [
                        ClickOutsideDirective
                    ],
                    imports: [
                        CommonModule,
                        FormsModule
                    ],
                    exports: [
                        PlTableComponent,
                        PlDatepickerComponent,
                        PlInputComponent,
                    ],
                    entryComponents: [
                        PlTableCellRendererComponent
                    ]
                },] }
    ];
    return NgParallacticUiModule;
}());

export { NgParallacticUiModule, ClickOutsideDirective as ɵa, PlTableComponent as ɵb, PlTableCellService as ɵc, PlTableRowComponent as ɵd, PlTableCellComponent as ɵe, PlTableCellDirective as ɵf, PlTableCellRendererComponent as ɵg, PlTableHeaderComponent as ɵh, PlTableHeaderCellComponent as ɵi, PlDatepickerComponent as ɵj, PlDatepickerService as ɵk, PlDatepickerOverlayComponent as ɵl, PlDatepickerCalendarComponent as ɵm, PlDatepickerHeaderComponent as ɵn, PlInputComponent as ɵo };
//# sourceMappingURL=ng-parallactic-ui.js.map
